/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.timers;

import java.lang.ref.WeakReference;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.netbeans.junit.NbTestCase;

/**
 *
 * @author Petr Hrebejk
 */
public class InstanceWatcherTest extends NbTestCase {

    public InstanceWatcherTest(String testName) {
    super(testName);
    }

    public void testFiring() throws Exception {
        System.out.println("addChangeListener");

        QueueListener listener = new QueueListener();
        InstanceWatcher iw = new InstanceWatcher();
        
        iw.addChangeListener(listener);
        
        Integer ts1 = new Integer( 20 );
        iw.add( ts1 );
               
        WeakReference tmp; // For forcing GC
        
        tmp = new WeakReference<Object>( new Object() );
        assertGC( "", tmp );
        
        assertEquals( "There should be no change in the queue", 0, listener.changeCount );
        
        ts1 = null; // Remove hard reference
        
        tmp = new WeakReference<Object>( new Object() );        
        assertGC( "", tmp ); // Do garbage collect
        
        assertEquals( "There should be one change in the queue", 1, listener.changeCount );
                
    }

    
    private static class QueueListener implements ChangeListener {
        
        int changeCount;
        
        public void stateChanged( ChangeEvent e ) {
            changeCount ++;
        }
        
    }
    
}